import * as React from 'react'
import Link from 'next/link'
import styled from 'styled-components'
import NavLink from '../NavLink/NavLink'

const StyledLink = styled('a')({
  display: 'block',
  textDecoration: 'none',
})

const Container = styled('span')({
  display: 'flex',
  alignItems: 'center',
  padding: '0 1rem',
  height: '100%',
  margin: '0 0 0 auto',
  boxSizing: 'border-box',
  '@media (min-width: 720px)': {
    maxWidth: 'var(--max-width)',
  },
})

const HeaderContainer = styled(Container)({
  marginTop: '2rem',
})

const NavContainer = styled('div')({

})

const Nav = ({
  data,
}) => {
  if (Array.isArray(data)) {
    return (
      <NavContainer>
        {
          data.map(d => (
            <Nav
              data={d}
            />
          ))
        }
      </NavContainer>
    )
  }
  if (typeof data === 'object') {
    const [entry] = Object.entries(data)
    const [key, value] = entry
    if (Array.isArray(value)) {
      return (
        <NavContainer>
          <HeaderContainer>
            {key}
          </HeaderContainer>
          {
            value.map(v => (
              <Nav
                data={v}
              />
            ))
          }
        </NavContainer>
      )
    }
    return (
      <Link
        href={value}
        passHref
      >
        <NavLink
          title={key}
          enclosingComponent={Container}
        />
      </Link>
    )
  }
  return (
    <Link
      href={data}
      passHref
    >
      <NavLink
        title={data}
        enclosingComponent={Container}
      />
    </Link>
  )
}

export default Nav
