import * as React from 'react'
import {
  LiveProvider,
  LiveEditor,
  LivePreview
} from 'react-live'
import styled from 'styled-components'

const Figure = styled('figure')({
  margin: 0,
})

const StyledLiveEditor = styled(LiveEditor)({
  lineHeight: 1.125,
  fontFamily: 'var(--font-family-monospace), monospace !important',
  marginTop: '1rem',
})

const Playground = ({
  components,
  code,
  label,
}) => {
  const indentLevel = code.indexOf('<') - 1
  const normalizedCode = code
    .split('\n')
    .map((s: string) => s.slice(indentLevel))
    .filter((s: string) => s.trim().length > 0)
    .join('\n')

  return (
    <Figure>
      {
        label
        && (
          <figcaption>
            {label}
          </figcaption>
        )
      }
      <div>
        <LiveProvider code={normalizedCode} scope={{
          ...components,
        }}>
          <LivePreview />
          <StyledLiveEditor />
        </LiveProvider>
      </div>
    </Figure>
  )
}

export default Playground
